@@ -1,13 +1,4 @@
-"""
-
-Reference: Weiping Song et al. "AutoInt: Automatic Feature Interaction Learning via Self-Attentive Neural Networks"
-           (https://arxiv.org/pdf/1810.11921.pdf)
-
-author: massquantity
-
-"""
-from tensorflow.keras.initializers import truncated_normal
-
+"""Implementation of AutoInt."""
 from ..bases import ModelMeta, TfBase
 from ..tfops import (
     dropout_config,
@@ -16,7 +7,6 @@ from ..tfops import (
     tf,
     tf_dense,
 )
-from ..training import TensorFlowTrainer
 from ..utils.validate import (
     check_dense_values,
     check_multi_sparse,
@@ -29,6 +19,62 @@ from ..utils.validate import (
 
 
 class AutoInt(TfBase, metaclass=ModelMeta):
+    """AutoInt algorithm.
+
+    Parameters
+    ----------
+    task : {'rating', 'ranking'}
+        Recommendation task. See :ref:`Task`.
+    data_info : :class:`~libreco.data.DataInfo` object
+        Object that contains useful information for training and inference.
+    loss_type : {'cross_entropy', 'focal'}, default: 'cross_entropy'
+        Loss for model training.
+    embed_size: int, default: 16
+        Vector size of embeddings.
+    n_epochs: int, default: 10
+        Number of epochs for training.
+    lr : float, default 0.001
+        Learning rate for training.
+    lr_decay : bool, default: False
+        Whether to use learning rate decay.
+    epsilon : float, default: 1e-5
+        A small constant added to the denominator to improve numerical stability in
+        Adam optimizer.
+        According to the `official comment <https://github.com/tensorflow/tensorflow/blob/v1.15.0/tensorflow/python/training/adam.py#L64>`_,
+        default value of `1e-8` for `epsilon` is generally not good, so here we choose `1e-5`.
+        Users can try tuning this hyperparameter if the training is unstable.
+    reg : float or None, default: None
+        Regularization parameter, must be non-negative or None.
+    batch_size : int, default: 256
+        Batch size for training.
+    num_neg : int, default: 1
+        Number of negative samples for each positive sample, only used in `ranking` task.
+    use_bn : bool, default: True
+        Whether to use batch normalization.
+    dropout_rate : float or None, default: None
+        Probability of an element to be zeroed. If it is None, dropout is not used.
+    att_embed_size : int, list of int or tuple of (int,), default: (8, 8, 8)
+        Embedding size in each attention layer. If it is `int`, one layer is used.
+    num_heads : int, default: 2
+        Number of heads in multi-head attention.
+    use_residual : bool, default: True
+        Whether to use residual layer.
+    multi_sparse_combiner : {'normal', 'mean', 'sum', 'sqrtn'}, default: 'sqrtn'
+        Options for combining `multi_sparse` features.
+    seed : int, default: 42
+        Random seed.
+    lower_upper_bound : tuple or None, default: None
+        Lower and upper score bound for `rating` task.
+    tf_sess_config : dict or None, default: None
+        Optional TensorFlow session config, see `ConfigProto options
+        <https://github.com/tensorflow/tensorflow/blob/v2.10.0/tensorflow/core/protobuf/config.proto#L431>`_.
+
+    References
+    ----------
+    *Weiping Song et al.* `AutoInt: Automatic Feature Interaction Learning via Self-Attentive Neural Networks
+    <https://arxiv.org/pdf/1810.11921.pdf>`_.
+    """
+
     user_variables = ["user_feat"]
     item_variables = ["item_feat"]
     sparse_variables = ["sparse_feat"]
@@ -37,13 +83,10 @@ class AutoInt(TfBase, metaclass=ModelMeta):
     def __init__(
         self,
         task,
-        data_info=None,
+        data_info,
         loss_type="cross_entropy",
         embed_size=16,
-        att_embed_size=None,
-        num_heads=2,
-        use_residual=True,
-        n_epochs=20,
+        n_epochs=10,
         lr=0.001,
         lr_decay=False,
         epsilon=1e-5,
@@ -52,25 +95,32 @@ class AutoInt(TfBase, metaclass=ModelMeta):
         num_neg=1,
         use_bn=True,
         dropout_rate=None,
-        hidden_units="128,64,32",
+        att_embed_size=(8, 8, 8),
+        num_heads=2,
+        use_residual=True,
         multi_sparse_combiner="sqrtn",
         seed=42,
         lower_upper_bound=None,
         tf_sess_config=None,
-        with_training=True,
     ):
         super().__init__(task, data_info, lower_upper_bound, tf_sess_config)
 
         self.all_args = locals()
+        self.loss_type = loss_type
         self.embed_size = embed_size
-        # 'att_embed_size' also decides the num of attention layer
+        self.n_epochs = n_epochs
+        self.lr = lr
+        self.lr_decay = lr_decay
+        self.epsilon = epsilon
+        self.reg = reg_config(reg)
+        self.batch_size = batch_size
+        self.num_neg = num_neg
+        self.use_bn = use_bn
+        self.dropout_rate = dropout_config(dropout_rate)
+        # `att_embed_size` also decides the num of attention layer
         self.att_embed_size, self.att_layer_num = self._att_config(att_embed_size)
         self.num_heads = num_heads
         self.use_residual = use_residual
-        self.reg = reg_config(reg)
-        self.use_bn = use_bn
-        self.dropout_rate = dropout_config(dropout_rate)
-        self.hidden_units = list(map(int, hidden_units.split(",")))
         self.seed = seed
         self.sparse = check_sparse_indices(data_info)
         self.dense = check_dense_values(data_info)
@@ -85,21 +135,8 @@ class AutoInt(TfBase, metaclass=ModelMeta):
             )
         if self.dense:
             self.dense_field_size = dense_field_size(data_info)
-        self._build_model()
-        if with_training:
-            self.trainer = TensorFlowTrainer(
-                self,
-                task,
-                loss_type,
-                n_epochs,
-                lr,
-                lr_decay,
-                epsilon,
-                batch_size,
-                num_neg,
-            )
 
-    def _build_model(self):
+    def build_model(self):
         tf.set_random_seed(self.seed)
         self.labels = tf.placeholder(tf.float32, shape=[None])
         self.is_training = tf.placeholder_with_default(False, shape=[])
@@ -126,13 +163,13 @@ class AutoInt(TfBase, metaclass=ModelMeta):
         user_feat = tf.get_variable(
             name="user_feat",
             shape=[self.n_users + 1, self.embed_size],
-            initializer=truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
         item_feat = tf.get_variable(
             name="item_feat",
             shape=[self.n_items + 1, self.embed_size],
-            initializer=truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -152,7 +189,7 @@ class AutoInt(TfBase, metaclass=ModelMeta):
         sparse_feat = tf.get_variable(
             name="sparse_feat",
             shape=[self.sparse_feature_size, self.embed_size],
-            initializer=truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -184,7 +221,7 @@ class AutoInt(TfBase, metaclass=ModelMeta):
         dense_feat = tf.get_variable(
             name="dense_feat",
             shape=[self.dense_field_size, self.embed_size],
-            initializer=truncated_normal(0.0, 0.01),
+            initializer=tf.glorot_uniform_initializer(),
             regularizer=self.reg,
         )
 
@@ -203,26 +240,26 @@ class AutoInt(TfBase, metaclass=ModelMeta):
         queries = tf_dense(
             units=multi_embed_size,
             activation=None,
-            kernel_initializer=truncated_normal(0.0, 0.01),
+            kernel_initializer=tf.glorot_uniform_initializer(),
             use_bias=False,
         )(inputs)
         keys = tf_dense(
             units=multi_embed_size,
             activation=None,
-            kernel_initializer=truncated_normal(0.0, 0.01),
+            kernel_initializer=tf.glorot_uniform_initializer(),
             use_bias=False,
         )(inputs)
         values = tf_dense(
             units=multi_embed_size,
             activation=None,
-            kernel_initializer=truncated_normal(0.0, 0.01),
+            kernel_initializer=tf.glorot_uniform_initializer(),
             use_bias=False,
         )(inputs)
         if self.use_residual:
             residual = tf_dense(
                 units=multi_embed_size,
                 activation=None,
-                kernel_initializer=truncated_normal(0.0, 0.01),
+                kernel_initializer=tf.glorot_uniform_initializer(),
                 use_bias=False,
             )(inputs)
 
