@@ -65,8 +65,8 @@ def predict_tf_feat(model, user, item, feats, cold_start, inner_id):
             item_indices=item_indices,
             sparse_indices=sparse_indices,
             dense_values=dense_values,
-            user_interacted_seq=model.user_last_interacted[user_indices],
-            user_interacted_len=model.last_interacted_len[user_indices],
+            user_interacted_seq=model.recent_seqs[user_indices],
+            user_interacted_len=model.recent_seq_lens[user_indices],
             is_training=False,
         )
     else:
@@ -106,8 +106,8 @@ def predict_data_with_feats(
                 item_indices=item_indices,
                 sparse_indices=sparse_indices,
                 dense_values=dense_values,
-                user_interacted_seq=model.user_last_interacted[user_indices],
-                user_interacted_len=model.last_interacted_len[user_indices],
+                user_interacted_seq=model.recent_seqs[user_indices],
+                user_interacted_len=model.recent_seq_lens[user_indices],
                 is_training=False,
             )
         else:
