@@ -1,6 +1,6 @@
-from .knn import save_knn
 from .embed import save_embed, save_faiss_index
-from .redis import knn2redis, embed2redis, tf2redis
+from .knn import save_knn
+from .redis import embed2redis, knn2redis, tf2redis
 from .tfmodel import save_tf
 
 __all__ = [
